@@ -33,6 +33,7 @@ from saltfactories.utils.processes import Popen
 from saltfactories.utils.processes import ProcessResult
 from saltfactories.utils.processes import ShellResult
 from saltfactories.utils.processes import terminate_process
+from saltfactories.utils.processes import terminate_process_list
 
 log = logging.getLogger(__name__)
 
@@ -351,12 +352,16 @@ class DaemonFactory(SubprocessFactoryBase):
     after_start_callbacks = attr.ib(repr=False, hash=False, default=attr.Factory(list))
     after_terminate_callbacks = attr.ib(repr=False, hash=False, default=attr.Factory(list))
     extra_cli_arguments_after_first_start_failure = attr.ib(hash=False, default=attr.Factory(list))
+    listen_ports = attr.ib(init=False, repr=False, hash=False, default=attr.Factory(list))
 
     def __attrs_post_init__(self):
         super().__attrs_post_init__()
         if self.check_ports and not isinstance(self.check_ports, (list, tuple)):
             self.check_ports = [self.check_ports]
+        if self.check_ports:
+            self.listen_ports.extend(self.check_ports)
         self.register_after_start_callback(self._add_factory_to_stats_processes)
+        self.register_after_terminate_callback(self._terminate_processes_matching_listen_ports)
         self.register_after_terminate_callback(self._remove_factory_from_stats_processes)
 
     def register_before_start_callback(self, callback, *args, **kwargs):
@@ -430,9 +435,13 @@ class DaemonFactory(SubprocessFactoryBase):
                     log.warning("%s is no longer running", self)
                     self.terminate()
                     break
-                if self.run_start_checks(current_start_time, start_running_timeout) is False:
-                    time.sleep(1)
-                    continue
+                try:
+                    if self.run_start_checks(current_start_time, start_running_timeout) is False:
+                        time.sleep(1)
+                        continue
+                except FactoryNotStarted:
+                    self.terminate()
+                    break
                 log.info(
                     "The %s factory is running after %d attempts. Took %1.2f seconds",
                     self,
@@ -510,11 +519,12 @@ class DaemonFactory(SubprocessFactoryBase):
         checks_start_time = time.time()
         while time.time() <= timeout_at:
             if not self.is_running():
-                log.warning("%s is no longer running", self)
-                return False
+                raise FactoryNotStarted("{} is no longer running".format(self))
             if not check_ports:
                 break
             check_ports -= ports.get_connectable_ports(check_ports)
+            if check_ports:
+                time.sleep(0.5)
         else:
             log.error(
                 "Failed to check ports after %1.2f seconds for %s",
@@ -537,6 +547,34 @@ class DaemonFactory(SubprocessFactoryBase):
             display_name = self.get_display_name()
             self.factories_manager.stats_processes.pop(display_name, None)
 
+    def _terminate_processes_matching_listen_ports(self):
+        if not self.listen_ports:
+            return
+        # If any processes were not terminated and are listening on the ports
+        # we have set on listen_ports, terminate those processes.
+        found_processes = []
+        for process in psutil.process_iter(["connections"]):
+            for connection in process.connections():
+                if connection.status != psutil.CONN_LISTEN:
+                    # We only care about listening services
+                    continue
+                if connection.laddr.port in self.check_ports:
+                    found_processes.append(process)
+                    # We already found one connection, no need to check the others
+                    break
+        if found_processes:
+            log.debug(
+                "The following processes were found listening on ports %s: %s",
+                ", ".join([str(port) for port in self.listen_ports]),
+                found_processes,
+            )
+            terminate_process_list(found_processes, kill=True, slow_stop=False)
+        else:
+            log.debug(
+                "No astray processes were found listening on ports: %s",
+                ", ".join([str(port) for port in self.listen_ports]),
+            )
+
     def __enter__(self):
         if not self.is_running():
             raise RuntimeError(
@@ -855,6 +893,7 @@ class SaltDaemonFactory(SaltFactory, DaemonFactory):
         if not super().run_start_checks(started_at, timeout_at):
             return False
         if not self.event_listener:
+            log.debug("The 'event_listener' attribute is not set. Not checking events...")
             return True
 
         check_events = set(self.get_check_events())
@@ -864,11 +903,12 @@ class SaltDaemonFactory(SaltFactory, DaemonFactory):
         checks_start_time = time.time()
         while time.time() <= timeout_at:
             if not self.is_running():
-                log.info("%s is no longer running", self)
-                return False
+                raise FactoryNotStarted("{} is no longer running".format(self))
             if not check_events:
                 break
             check_events -= self.event_listener.get_events(check_events, after_time=started_at)
+            if check_events:
+                time.sleep(0.5)
         else:
             log.error(
                 "Failed to check events after %1.2f seconds for %s",
